package com.aelitis.azureus.core.peermanager.unchoker;

/**
 * Encapsulates data about the state of upload/download counters at some
 * point in history.  Useful for calculating actions to take based on recent
 * history of a peer.
 * @author Jarret
 *
 */
public class HistoricalDataPoint {
	long timestamp;
	long dataSentToPeer;
	long dataReceivedFromPeer;
	long dataAllocatedToPeer = -1;
	long dataPulledByPeer = -1;
	/**
	 * Make a data point using the given upload and download counts
	 * @param dataSentToPeer the total amount of data (payload) that has been sent to the peer
	 * @param dataReceivedFromPeer the total amount of data that has been received from the peer
	 */
	public HistoricalDataPoint(long dataSentToPeer, long dataReceivedFromPeer) {
		this.timestamp = System.currentTimeMillis();
		this.dataSentToPeer = dataSentToPeer;
		this.dataReceivedFromPeer = dataReceivedFromPeer;
	}
	public HistoricalDataPoint(long dataSentToPeer, long dataReceivedFromPeer, long dataAllocatedToPeer, long dataPulledByPeer) {
		this.timestamp = System.currentTimeMillis();
		this.dataSentToPeer = dataSentToPeer;
		this.dataReceivedFromPeer = dataReceivedFromPeer;
		this.dataAllocatedToPeer = dataAllocatedToPeer;
		this.dataPulledByPeer = dataPulledByPeer;
	}
	public long getDataAllocatedToPeer() {
		return dataAllocatedToPeer;
	}
	public long getDataPulledByPeer() {
		return dataPulledByPeer;
	}
	public long getDataReceivedFromPeer() {
		return dataReceivedFromPeer;
	}
	public long getDataSentToPeer() {
		return dataSentToPeer;
	}
	public long getTimestamp() {
		return timestamp;
	}
	
	public String toString(){
		if(dataAllocatedToPeer < 0 && dataPulledByPeer < 0)
			return "when " + timestamp + " rec: " + dataReceivedFromPeer + " sent: " + dataSentToPeer;
		else
			return "when " + timestamp + " rec: " + dataReceivedFromPeer + " sent: " + dataSentToPeer + 
				" alloc: " + dataAllocatedToPeer + " pulled: " + dataPulledByPeer;
	}
}
